#include < amxmodx >
#include < amxmisc >
#include < hamsandwich >
#include < engine >
#include < eas_core >

#define PLUGIN_AUTHOR "DoNii"
#define PLUGIN_NAME "Elite Admin System Ranks"

#if defined client_disconnected
#define client_disconnect client_disconnected
#endif

new const szVersion[ ] = "6.7.2";

enum _:Ranks
{
	Rank_Name[ 32 ],
	Rank_Flags[ MAX_FLAGS_LENGTH ]
}

enum ( += 15000 )
{
	TASK_HUD_ALIVE,
	TASK_HUD_DEAD
}

enum _:Config
{
	RANKS_FILE[ 32 ],
	USERS_FILE[ 32 ],
	CONFIGS_FILE[ 32 ],
	BANS_FILE[ 32 ],
	GAGS_FILE[ 32 ],
	LOGS_FILE[ 32 ],
	CVARS_FILE[ 32 ],
	SYSTEM_PREFIX[ 32 ],
	
	RANK_ALIVE_HUD_R[ 3 ],
	RANK_ALIVE_HUD_G[ 3 ],
	RANK_ALIVE_HUD_B[ 3 ],
	Float:RANK_ALIVE_HUD_X[ 3 ],
	Float:RANK_ALIVE_HUD_Y[ 3 ],
	RANKS_SHOW_UNRANKED,
	RANKS_HUD_ENABLED,

	RANK_DEAD_HUD_R[ 3 ],
	RANK_DEAD_HUD_G[ 3 ],
	RANK_DEAD_HUD_B[ 3 ],
	Float:RANK_DEAD_HUD_X[ 3 ],
	Float:RANK_DEAD_HUD_Y[ 3 ],
}

new g_iHudSync;
new Array:g_aRanks;
new g_szConfigsDir[ 128 ];
new g_iConfig[ Config ];

public plugin_end( )
{
	ArrayDestroy( g_aRanks );
}

public plugin_precache( )
{
	get_configsdir( g_szConfigsDir, charsmax( g_szConfigsDir ) );
	
	register_dictionary( "elite_admin_system.txt" );
	
	if( CheckConfigFile( ) )
	{
		server_cmd( "exec %s/amxx.cfg", g_szConfigsDir ); // executing this here since this is the first plugin
		
		ReadConfig( );
		CheckFilesExist( );
		
		g_aRanks = ArrayCreate( Ranks );
		ReadRanks( );
		
		g_iHudSync = CreateHudSyncObj( );
		
		if( g_iConfig[ RANKS_HUD_ENABLED ] )
		{
			RegisterHam( Ham_Spawn, "player", "@Ham_Spawn_Post", 1 );
			RegisterHam( Ham_Killed, "player", "@Ham_Killed_Post", 1 );
		}
	}
	
	else
	{
		server_print( "%L", LANG_PLAYER, "ERROR_MISSING" );
		pause( "a" );
	}
}

public plugin_init( )
{
	register_plugin( PLUGIN_NAME, szVersion, PLUGIN_NAME );
}

public plugin_natives( )
{
	register_library( "eas_ranks" );
	register_native( "eas_get_player_rank", "OnNativeGetPlayerRank" );
}

public OnNativeGetPlayerRank( )
{
	new szRank[ 32 ], iLen = get_param( 3 );
	GetPlayerRank( get_param( 1 ), szRank, iLen );
	
	set_string( 2, szRank, iLen );
}

@Ham_Spawn_Post( id )
{
	if( ! is_user_alive( id ) )
	{
		return HAM_IGNORED;
	}

	ClearSyncHud( id, g_iHudSync );
	set_task( 1.0, "AliveRanksHud", id + TASK_HUD_ALIVE, .flags="b" );
	
	return HAM_IGNORED;
}

@Ham_Killed_Post( iVictim )
{
	ClearSyncHud( iVictim, g_iHudSync );
	set_task( 1.0, "DeadRanksHud", iVictim + TASK_HUD_DEAD, .flags="b" );
}

public AliveRanksHud( id )
{
	id -= TASK_HUD_ALIVE;
	
	new szRank[ 32 ], szName[ MAX_NAME_LENGTH ];
	GetPlayerRank( id, szRank, charsmax( szRank ) )
	
	if( ! szRank[ 0 ] )
	{
		remove_task( id + TASK_HUD_ALIVE );
		return PLUGIN_HANDLED;
	}	
	
	get_user_name( id, szName, charsmax( szName ) );
	
	set_hudmessage( g_iConfig[ RANK_ALIVE_HUD_R ], g_iConfig[ RANK_ALIVE_HUD_G ], g_iConfig[ RANK_ALIVE_HUD_B ], g_iConfig[ RANK_ALIVE_HUD_X ], g_iConfig[ RANK_ALIVE_HUD_Y ], 0, 0.0, 1.1 )
	ShowSyncHudMsg( id, g_iHudSync, "%L: %s | %L: %s", id, "NAME", szName, id, "RANK", szRank );

	return PLUGIN_CONTINUE;
}

public DeadRanksHud( id )
{
	id -= TASK_HUD_DEAD;

	new iSpecId = entity_get_int( id, EV_INT_iuser2 );
	
	if( is_user_alive( iSpecId ) )
	{
		new szRank[ 32 ], szName[ MAX_NAME_LENGTH ];
		get_user_name( iSpecId, szName, charsmax( szName ) );
		
		GetPlayerRank( id, szRank, charsmax( szRank ) )
		
		if( ! szRank[ 0 ] )
		{
			remove_task( id + TASK_HUD_DEAD );
			return PLUGIN_HANDLED;
		}	
		
		set_hudmessage( g_iConfig[ RANK_DEAD_HUD_R ], g_iConfig[ RANK_DEAD_HUD_G ], g_iConfig[ RANK_DEAD_HUD_B ], g_iConfig[ RANK_DEAD_HUD_X ], g_iConfig[ RANK_DEAD_HUD_Y ], 0, 0.0, 1.1 )
		ShowSyncHudMsg( id, g_iHudSync, "%L: %s | %L: %s", id, "SPECTATING_NAME", szName, id, "RANK", szRank );
	}
	return PLUGIN_CONTINUE;
}

GetPlayerRank( id, szRank[ ], iSize )
{
	new iFlags = get_user_flags( id );
	
	new szFlags[ 32 ];
	get_flags( iFlags, szFlags, charsmax( szFlags ) );
	
	new eRanks[ Ranks ], j = -1;
	for( new i; i < ArraySize( g_aRanks ); i++ )
	{
		ArrayGetArray( g_aRanks, i, eRanks );
		
		if( CharsInString( szFlags, charsmax( szFlags ) ) >= CharsInString( eRanks[ Rank_Flags ], charsmax( eRanks[ Rank_Flags ] ) ) && contain_all( eRanks[ Rank_Flags ], charsmax( eRanks[ Rank_Flags ] ), szFlags, charsmax( szFlags ) ) )
		{
			j = i;
		}
	}
	
	if( j == -1 )
	{
		if( g_iConfig[ RANKS_SHOW_UNRANKED ] )
		{
			new szBuffer[ 32 ];
			formatex( szBuffer, charsmax( szBuffer ), "%L", id, "NO_RANK" );
			
			copy( szRank, iSize, szBuffer );
		}
		
		else
		{
			copy( szRank, iSize, "" );
		}
	}
	
	else
	{
		arrayset( eRanks, 0, sizeof( eRanks ) );
		ArrayGetArray( g_aRanks, j, eRanks );
		
		copy( szRank, iSize, eRanks[ Rank_Name ] );
	}
}

ReadRanks( )
{
	new szFile[ 128 ], szData[ 64 ];
	
	formatex( szFile, charsmax( szFile ), "%s/%s", g_szConfigsDir, g_iConfig[ RANKS_FILE ] );
	
	new iFile = fopen( szFile, "rt" );
	
	if( iFile )
	{
		while( fgets( iFile, szData, charsmax( szData ) ) )
		{   
			trim( szData );
			remove_quotes( szData );
			
			switch( szData[ 0 ] )
			{
			case EOS, '#', ';', '/', '\': 
				{
					continue;
				}

			default:
				{
					new szKey[ 32 ], szValue[ 32 ];
					strtok( szData, szKey, charsmax( szKey ), szValue, charsmax( szValue ), '=' );
					trim( szKey ); 
					trim( szValue );
					
					remove_quotes( szKey );
					remove_quotes( szValue );
					
					if( ! szValue[ 0 ] || ! szKey[ 0 ] )
					{
						continue;
					}
					
					new eRanks[ Ranks ];
					copy( eRanks[ Rank_Name ], charsmax( eRanks[ Rank_Name ] ), szKey );
					copy( eRanks[ Rank_Flags ], charsmax( eRanks[ Rank_Flags ] ), szValue );
					
					ArrayPushArray( g_aRanks, eRanks );
				}
			}
		}
		fclose( iFile );
	}
}

public contain_all( szStringSrc[ ], iSize1, szStringCompare[ ], iSize2 )
{
	new bool:bEqual;

	for( new i; i < iSize1; i++ )
	{
		for( new j; j < iSize2; j++ )
		{
			if( szStringSrc[ i ] == szStringCompare[ j ] )
			{
				bEqual = true;
				break;
			}
			
			else
			{
				bEqual = false;
			}
		}
		
		if( ! bEqual )
		break;
	}
	
	return bEqual;
}

public CharsInString( szString[ ], iSize )
{
	new iCount;

	for( new i; i < iSize; i++ )
	{
		if( szString[ i ] )
		{
			iCount++;
		}
	}
	return iCount;
}

ReadConfig( )
{
	new szFile[ 128 ], szData[ 64 ];
	
	formatex( szFile, charsmax( szFile ), "%s/%s", g_szConfigsDir, g_szConfigFile );
	
	new iFile = fopen( szFile, "rt" );
	
	if( iFile )
	{
		while( fgets( iFile, szData, charsmax( szData ) ) )
		{    
			trim( szData );
			
			switch( szData[ 0 ] )
			{
			case EOS, '#', ';', '/', '\': 
				{
					continue;
				}

			default:
				{
					new szKey[ 32 ], szValue[ 64 ];
					strtok( szData, szKey, charsmax( szKey ), szValue, charsmax( szValue ), '=' );
					trim( szKey ); 
					trim( szValue );
					
					remove_quotes( szKey );
					remove_quotes( szValue );
					
					if( ! szValue[ 0 ] || ! szKey[ 0 ] )
					{
						continue;
					}
					
					if( equal( szKey, "RANKS_FILE" ) )
					{
						copy( g_iConfig[ RANKS_FILE ], charsmax( g_iConfig[ RANKS_FILE ] ), szValue );
					}
					
					else if( equal( szKey, "GAGS_FILE" ) )
					{
						copy( g_iConfig[ GAGS_FILE ], charsmax( g_iConfig[ GAGS_FILE ] ), szValue );
					}
					
					else if( equal( szKey, "BANS_FILE" ) )
					{
						copy( g_iConfig[ BANS_FILE ], charsmax( g_iConfig[ BANS_FILE ] ), szValue );
					}
					
					else if( equal( szKey, "LOGS_FILE" ) )
					{
						copy( g_iConfig[ LOGS_FILE ], charsmax( g_iConfig[ LOGS_FILE ] ), szValue );
					}
					
					else if( equal( szKey, "CONFIGS_FILE" ) )
					{
						copy( g_iConfig[ CONFIGS_FILE ], charsmax( g_iConfig[ CONFIGS_FILE ] ), szValue );
					}
					
					else if( equal( szKey, "USERS_FILE" ) )
					{
						copy( g_iConfig[ USERS_FILE ], charsmax( g_iConfig[ USERS_FILE ] ), szValue );
					}
					
					else if( equal( szKey, "CVARS_FILE" ) )
					{
						copy( g_iConfig[ CVARS_FILE ], charsmax( g_iConfig[ CVARS_FILE ] ), szValue );
					}
					
					else if( equal( szKey, "SYSTEM_PREFIX" ) )
					{
						copy( g_iConfig[ SYSTEM_PREFIX ], charsmax( g_iConfig[ SYSTEM_PREFIX ] ), szValue );
					}
					
					else if( equal( szKey, "RANK_ALIVE_HUD_R" ) )
					{
						g_iConfig[ RANK_ALIVE_HUD_R ] = str_to_num( szValue );
					}
					
					else if( equal( szKey, "RANK_ALIVE_HUD_G" ) )
					{
						g_iConfig[ RANK_ALIVE_HUD_G ] = str_to_num( szValue );
					}
					
					else if( equal( szKey, "RANK_ALIVE_HUD_B" ) )
					{
						g_iConfig[ RANK_ALIVE_HUD_B ] = str_to_num( szValue );
					}
					
					else if( equal( szKey, "RANK_ALIVE_HUD_X" ) )
					{
						g_iConfig[ RANK_ALIVE_HUD_X ] = _:str_to_float( szValue );
					}
					
					else if( equal( szKey, "RANK_ALIVE_HUD_Y" ) )
					{
						g_iConfig[ RANK_ALIVE_HUD_Y ] = _:str_to_float( szValue );
					}
					
					else if( equal( szKey, "RANK_DEAD_HUD_R" ) )
					{
						g_iConfig[ RANK_DEAD_HUD_R ] = str_to_num( szValue );
					}
					
					else if( equal( szKey, "RANK_DEAD_HUD_G" ) )
					{
						g_iConfig[ RANK_DEAD_HUD_G ] = str_to_num( szValue );
					}
					
					else if( equal( szKey, "RANK_DEAD_HUD_B" ) )
					{
						g_iConfig[ RANK_DEAD_HUD_B ] = str_to_num( szValue );
					}
					
					else if( equal( szKey, "RANK_DEAD_HUD_X" ) )
					{
						g_iConfig[ RANK_DEAD_HUD_X ] = _:str_to_float( szValue );
					}
					
					else if( equal( szKey, "RANK_DEAD_HUD_Y" ) )
					{
						g_iConfig[ RANK_DEAD_HUD_Y ] = _:str_to_float( szValue );
					}
					
					else if( equal( szKey, "RANKS_SHOW_UNRANKED" ) )
					{
						g_iConfig[ RANKS_SHOW_UNRANKED ] = str_to_num( szValue );
					}
					
					else if( equal( szKey, "RANKS_HUD_ENABLED" ) )
					{
						g_iConfig[ RANKS_HUD_ENABLED ] = str_to_num( szValue );
					}
				}
			}
		}
		fclose( iFile );
	}
}

CheckConfigFile( )
{
	new szConfigsFile[ 256 ];
	formatex( szConfigsFile, charsmax( szConfigsFile ), "%s/%s", g_szConfigsDir, g_szConfigFile );

	if( ! file_exists( szConfigsFile ) )
	{
		new iFile = fopen( szConfigsFile, "w" );
		fclose( iFile );
		
		return 0;
	}
	return 1;
}

CheckFilesExist( )
{
	new szUsersFile[ 256 ], szBansFile[ 256 ], szGagsFile[ 256 ], szRanksFile[ 256 ], szCvarsFile[ 256 ];

	formatex( szUsersFile, charsmax( szUsersFile ), "%s/%s", g_szConfigsDir, g_iConfig[ USERS_FILE ] );
	formatex( szBansFile, charsmax( szBansFile ), "%s/%s", g_szConfigsDir, g_iConfig[ BANS_FILE ] );
	formatex( szGagsFile, charsmax( szGagsFile ), "%s/%s", g_szConfigsDir, g_iConfig[ GAGS_FILE ] );
	formatex( szRanksFile, charsmax( szRanksFile ), "%s/%s", g_szConfigsDir, g_iConfig[ RANKS_FILE ] );
	formatex( szCvarsFile, charsmax( szCvarsFile ), "%s/%s", g_szConfigsDir, g_iConfig[ CVARS_FILE ] );
	
	new szDate[ MAX_DATE_LENGTH ], szNewLogsFile[ 256 ];
	get_time( "%m-%d-%Y", szDate, charsmax( szDate ) );
			
	formatex( szNewLogsFile, charsmax( szNewLogsFile ), "%s_%s.ini", g_iConfig[ LOGS_FILE ], szDate );
	
	new iFile;
	if( ! file_exists( szNewLogsFile ) )
	{	
		iFile = fopen( szNewLogsFile, "w" );
		fclose( iFile );
		
		log_to_file( szNewLogsFile, "%L", LANG_PLAYER, "LOGS_FILE_CREATED" );
	}
	
	if( ! file_exists( szUsersFile ) )
	{
		iFile = fopen( szUsersFile, "w" );
		fclose( iFile );
		
		log_to_file( szNewLogsFile, "%L", LANG_PLAYER, "USERS_FILE_CREATED" );
	}

	if( ! file_exists( szBansFile ) )
	{
		iFile = fopen( szBansFile, "w" );
		fclose( iFile );
		
		log_to_file( szNewLogsFile, "%L", LANG_PLAYER, "BANS_FILE_CREATED" );
	}

	if( ! file_exists( szGagsFile ) )
	{
		iFile = fopen( szGagsFile, "w" );
		fclose( iFile );
		
		log_to_file( szNewLogsFile, "%L", LANG_PLAYER, "GAGS_FILE_CREATED" );
	}
	
	if( ! file_exists( szRanksFile ) )
	{
		iFile = fopen( szRanksFile, "w" );
		fclose( iFile );
		
		log_to_file( szNewLogsFile, "%L", LANG_PLAYER, "RANKS_FILE_CREATED" );
	}
	
	if( ! file_exists( szCvarsFile ) )
	{
		iFile = fopen( szCvarsFile, "w" );
		fclose( iFile );
		
		log_to_file( szNewLogsFile, "%L", LANG_PLAYER, "CVARS_FILE_CREATED" );
	}
	return 1;
}